package sun.security.krb5.internal;

import java.math.BigInteger;
import sun.security.util.DerOutputStream;
import java.util.Vector;
import java.io.IOException;
import sun.security.krb5.Asn1Exception;
import sun.security.util.DerValue;
import sun.security.krb5.EncryptionKey;

public class EncAPRepPart
{
    public KerberosTime ctime;
    public int cusec;
    EncryptionKey subKey;
    Integer seqNumber;
    
    public EncAPRepPart(KerberosTime ctime, int cusec, EncryptionKey subKey, Integer seqNumber) {
        this.ctime = ctime;
        this.cusec = cusec;
        this.subKey = subKey;
        this.seqNumber = seqNumber;
    }
    
    public EncAPRepPart(byte[] encoding) throws Asn1Exception, IOException {
        this.init(new DerValue(encoding));
    }
    
    public EncAPRepPart(DerValue derValue) throws Asn1Exception, IOException {
        this.init(derValue);
    }
    
    private void init(DerValue derValue) throws Asn1Exception, IOException {
        if ((derValue.getTag() & 0x1F) != 0x1B || !derValue.isApplication() || !derValue.isConstructed()) {
            throw new Asn1Exception(906);
        }
        DerValue derValue2 = derValue.getData().getDerValue();
        if (derValue2.getTag() != 48) {
            throw new Asn1Exception(906);
        }
        this.ctime = KerberosTime.parse(derValue2.getData(), (byte)0, true);
        DerValue derValue3 = derValue2.getData().getDerValue();
        if ((derValue3.getTag() & 0x1F) == 0x1) {
            this.cusec = derValue3.getData().getBigInteger().intValue();
            if (derValue2.getData().available() > 0) {
                this.subKey = EncryptionKey.parse(derValue2.getData(), (byte)2, true);
            }
            else {
                this.subKey = null;
                this.seqNumber = null;
            }
            if (derValue2.getData().available() > 0) {
                DerValue derValue4 = derValue2.getData().getDerValue();
                if ((derValue4.getTag() & 0x1F) != 0x3) {
                    throw new Asn1Exception(906);
                }
                this.seqNumber = new Integer(derValue4.getData().getBigInteger().intValue());
            }
            else {
                this.seqNumber = null;
            }
            if (derValue2.getData().available() > 0) {
                throw new Asn1Exception(906);
            }
            return;
        }
        else {
            throw new Asn1Exception(906);
        }
    }
    
    public byte[] asn1Encode() throws Asn1Exception, IOException {
        Vector<DerValue> vector = new Vector();
        DerOutputStream derOutputStream = new DerOutputStream();
        vector.addElement(new DerValue(DerValue.createTag((byte)(-128), true, (byte)0), this.ctime.asn1Encode()));
        derOutputStream.putInteger(BigInteger.valueOf((long)this.cusec));
        vector.addElement(new DerValue(DerValue.createTag((byte)(-128), true, (byte)1), derOutputStream.toByteArray()));
        if (this.subKey != null) {
            vector.addElement(new DerValue(DerValue.createTag((byte)(-128), true, (byte)2), this.subKey.asn1Encode()));
        }
        if (this.seqNumber != null) {
            DerOutputStream derOutputStream2 = new DerOutputStream();
            derOutputStream2.putInteger(BigInteger.valueOf((long)this.seqNumber));
            vector.addElement(new DerValue(DerValue.createTag((byte)(-128), true, (byte)3), derOutputStream2.toByteArray()));
        }
        DerValue[] anArray = new DerValue[vector.size()];
        vector.copyInto(anArray);
        DerOutputStream o = new DerOutputStream();
        o.putSequence(anArray);
        DerOutputStream o2 = new DerOutputStream();
        o2.write(DerValue.createTag((byte)64, true, (byte)27), o);
        return o2.toByteArray();
    }
    
    public final EncryptionKey getSubKey() {
        return this.subKey;
    }
    
    public final Integer getSeqNumber() {
        return this.seqNumber;
    }
}
